#include "cpmt.h"
#include "Undoables.h"

#include "cobj.h"
#include "cac2obj.h"
#include "cstrlist.h"

// Global PMT object
CPmt *pmt = NULL;

//---------------------------------------------------------------------------------------
// Load PMT geometry
static void ddhpmt_load()
{

   // Get the file
   char *fname;
   {
      char *filter[] = { "OBJ files", ".obj", "All files", "*", NULL };
      fname = ac_get_import_filename( "Import Poser Morph Target from OBJ", filter );
      if(( fname == NULL ) || ( *fname =='\0' ))
      {
         return;
      }
   }
#ifdef WINDOWS
   ac_unbackslash_string( fname );
#endif

   // Load the OBJ file
   CObj obj;
   try
   {
      debugf( "pmt : loading file %s\n", fname );
      obj.LoadFromFile( fname );
      debugf( "pmt : file %s loaded\n", fname );
   }
   catch( CExcept &e )
   {
      message_dialog( "pmt : %s", e.text );
      debugf( "pmt : %s", e.text );
   }

   // Add new materials
   debugf( "pmt : Adding %d materials\n", obj.mMCount );
   for( int ix = 0; ix < obj.mMCount; ++ix )
   {
      CObjMat *om = &( obj.mM[ix] );
      ACMaterialTemplate *mt = new ACMaterialTemplate();
      mt->ambient.r = om->mAmbient[0];
      mt->ambient.g = om->mAmbient[1];
      mt->ambient.b = om->mAmbient[2];
      mt->rgb.r = om->mDiffuse[0];
      mt->rgb.g = om->mDiffuse[1];
      mt->rgb.b = om->mDiffuse[2];
      mt->specular.r = om->mSpecular[0];
      mt->specular.g = om->mSpecular[1];
      mt->specular.b = om->mSpecular[2];
      mt->emissive.r = 0;
      mt->emissive.g = 0;
      mt->emissive.b = 0;
      mt->shininess = om->mShininess;
      mt->transparency = min( 1.0f, max( 0.0f, 1.0f - om->mTransparency ));
      // OLD method - doesn't account for dupes
      //ACMaterial *m = ac_new_material_from_template( mt );
      //ac_material_set_name( m, om->mName );
      //om->mTag = ac_palette_append_material( m );
      // NEW method - accounts for dupes
      om->mTag = ac_palette_get_new_material_index( mt );
      ACMaterial *m = ac_palette_get_material( om->mTag );
      ac_material_set_name( m, om->mName );
   }

   // Allocate a vertex list
   pmt->AllocateVerts( obj.mVCount );

   // Create ac3d object
   ACObject *o = ac_new_object();
   object_set_name( o, "Poser Morph Target" );

   // Add vertices to object
   for( int ix = 0; ix < obj.mVCount; ++ix )
   {
      pmt->mV[ix] = new_vertex_set( obj.mV[ix].x, obj.mV[ix].y, obj.mV[ix].z );
      object_add_vertex_head( o, pmt->mV[ix] );
   }

   // Add surfaces to object
   for( int ix = 0; ix < obj.mFCount; ++ix )
   {
      Surface *s = new_surface();
      if( obj.mF[ix].mSmooth )
      {
         surface_set_shading( s, TRUE );
      }
      else
      {
         surface_set_shading( s, FALSE );
      }
      surface_set_twosided( s, FALSE );
      if( obj.mF[ix].mIsLine )
      {
         surface_set_type( s, SURFACE_LINE );
      }
      for( CObjFaceVert *fv = obj.mF[ix].mHead; fv != NULL; fv = fv->mNext )
      {
         surface_add_vertex( s, pmt->mV[fv->mVidx], 0, 0 );
      }
      if( obj.mF[ix].mMatIndex > -1 )
      {
         s->col = obj.mM[obj.mF[ix].mMatIndex].mTag;
      }
      object_add_surface_head( o, s );
   }

   object_add_child( ac_get_world(), o );
   object_calc_normals_force( o );
   clear_selection();
   ac_select_object( o );
   fit_selected_all();
   redraw_all();
   display_status();

}
//---------------------------------------------------------------------------------------
// Save PMT geometry
static void ddhpmt_savepmt( bool vertsonly )
{

   // Ensure all of our vertices still exist
   if( pmt->mVCount == 0 )
   {
      message_dialog( "pmt : no PMT data found in memory - no morph targets have been loaded during this session?" );
      return;
   }

   // Init ac2obj - getting ready to write objects/vertices
   CAc2Obj ac2obj;
   bool hasOrphans = false;
   unsigned int numFound = 0;
   ac2obj.IdentifyAcVertices( numFound, hasOrphans );

   // Report missing vertices
   if( numFound != pmt->mVCount )
   {
      message_dialog( "pmt : could not find all of the vertices that were loaded with the original morph target.  Some vertices appear to have been deleted/misplaced." );
      ac2obj.ClearVerts();
      ac2obj.ClearFiles();
      return;
   }
   if( hasOrphans )
   {
      message_dialog( "pmt : geometry was found in your document that was not loaded as part of the original morph target.  Please remove this geometry before saving your work.  If you need the geometry intact, consider using the File -> Save Selection... command to save the geometry to a new file, then delete it from this document before exporting again." );
      ac2obj.ClearVerts();
      ac2obj.ClearFiles();
      return;
   }

   // Get the export filename
   char *fname;
   {
      char *filter[] = { "OBJ files", ".obj", "All files", "*", NULL };
      fname = ac_get_export_filename( "Export Poser Morph Target to OBJ", filter );
      if(( fname == NULL ) || ( *fname =='\0' ))
      {
         ac2obj.ClearVerts();
         ac2obj.ClearFiles();
         return;
      }
   }
#ifdef WINDOWS
   ac_unbackslash_string( fname );
#endif

   // Get the material filename
   if( strlen( fname ) > ( COBJ_MAXSTRLEN - 6 ))
   {
      message_dialog( "pmt : filename is too long" );
   }
   char mname[COBJ_MAXSTRLEN];
   strcpy( mname, fname );
   char *extloc = strrchr( mname, '.' );
   if( extloc == NULL )
   {
      strcat( mname, ".mtl" );
   }
   else
   {
      strcpy( extloc, ".mtl" );
   }

   // Save to the output file
   try
   {
      ac2obj.SaveToObjFile( fname, mname, vertsonly );
   }
   catch( CExcept &e )
   {
      message_dialog( "pmt : %s", e.text );
   }

   ac2obj.ClearVerts();
   ac2obj.ClearFiles();

}
//---------------------------------------------------------------------------------------
static void ddhpmt_save()
{
   ddhpmt_savepmt( false );
}
//---------------------------------------------------------------------------------------
static void ddhpmt_save_final()
{
   ddhpmt_savepmt( true );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   if( ac_get_version_number() < 5.0 - 0.00005 ||
       ( ac_get_version_number() < 6.0 - 0.00005 &&
         ac_get_version_revision() < 20.99999 ))
   {
      message_dialog( "Sorry, PMT works only in AC3D 5.0.21 and later." );
      return 1;
   }

   static bool firstTime = true;
   if( firstTime )
   {

      pmt = CPmt::GetSingleton();

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/pmt.tcl\"", path );
      ac_add_tools_menu_item( "PMT: Poser morph target assist...", "ddhpmt_dialog",
                              "Poser morph target assistant" );
      ac_add_command( "ddhpmt_load", ddhpmt_load );
      ac_add_command( "ddhpmt_save", ddhpmt_save );
      ac_add_command( "ddhpmt_save_final", ddhpmt_save_final );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "PMT - Poser Morph Target assistant - version 1.0.1 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "PMT - Poser Morph Target assistant.  Loads/saves OBJ files for use as Poser Morph targets." );
}
//---------------------------------------------------------------------------------------
